/*

  File FFT.cpp:

  contiene la definizione delle funzioni membro della classe FFT,
  che realizza la trasformata di Fourier diretta/inversa di un
  dataset reale/complesso 1D.
  
  Utilizza la libreria FFTW.

*/

#include "FFT.h"


/*

  FFT(): costruttore della classe. Inizializza i piani per la trasformata
  di Fourier diretta/inversa e i vettori per lo storage dei dati

*/

FFT::FFT( long dim )
{
  N  = dim;     // Dimensione della trasformata
  N2 = N/2;     // Frequenza di Nyquist
  datar = ( double * )fftw_malloc( sizeof( double ) * N );
  datac = ( fftw_complex * )fftw_malloc( sizeof( fftw_complex ) * ( N2 + 1 ) );
  planf = fftw_plan_dft_r2c_1d( N, datar, datac, FFTW_ESTIMATE|FFTW_DESTROY_INPUT );
  planb = fftw_plan_dft_c2r_1d( N, datac, datar, FFTW_ESTIMATE|FFTW_DESTROY_INPUT );
}


/*

  ~FFT(): distruttore della classe. Rilascia la memoria allocata per lo storage
  dei dato e distrugge i piani fftw

*/

FFT::~FFT( )
{
  fftw_destroy_plan( planf );
  fftw_destroy_plan( planb );
  fftw_free( datar );
  fftw_free( datac );
}


/*

  DT( data[N], anR[N/2+1], anI[N/2+1] ):
  calcola la trasformata di Fourier diretta (da spazio fisico a spazio spettrale)
  di un vettore di N dati reali (double), restituendo N/2+1 parti reali ed 
  immaginarie per i coefficienti di Fourier del vettore iniziale.

  Nota : la divisione finale per N dei dati serve perche' la fftw restituisce
          una trasformata NON-normalizzata! (vedi manuale fftw).

*/

void FFT::DT( double data[], double anR[], double anI[] )
{
  // Copia il vettore di input nello storage temporaneo "datar"

  for ( long i = 0; i < N; i++ )
    datar[i] = data[i];

  // Esegue la trasformata diretta: datar -> datac

  fftw_execute( planf );

  // Ricopia il vettore di output datac delle armoniche di Fourier in anR e anI

  for ( long i = 0; i <= N2; i++ )
    {
      anR[i] = datac[i][0] / N;
      anI[i] = datac[i][1] / N;
    }

}


/*

  IT( data[N], anR[N/2+1], anI[N/2+1] ):
  calcola la trasformata di Fourier inversa (da spazio spettrale a spazio fisico)
  di un vettore contenente le  N/2+1 parti reali ed immaginarie dei coefficienti 
  di Fourier, restituendo il dataset data di N valori reali.

  Nota: l'ordine della chiamata e' identico a quello di DT!

*/

void FFT::IT( double data[], double anR[], double anI[] )
{
  // Copia il vettore di input nello storage temporaneo "datac"

  for ( long i = 0; i <= N2; i++ )
    {
      datac[i][0] = anR[i];
      datac[i][1] = anI[i];
    }

  // Esegue la trasformata inversa: datac -> datar

  fftw_execute( planb );

  // Ricopia il vettore di output datar in data

  for ( long i = 0; i < N; i++ )
    data[i] = datar[i];

}
